<?php
/*
Plugin Name: Facebook Albums
Plugin URI: http://glamanate.com/wordpress/facebook-album/
Description: Facebook Albums allows you to plug a Facebook Page album into your Wordpress! Utilizes Lightbox2
Version: 1.0.5
Author: Matt Glaman
Author URI: http://glamanate.com


Copyright 2012  Matt Glaman  (email : nmd.matt@gmail.com)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

//Registers the short code to be placed within posts or pages.
add_shortcode( 'fbalbum', array('FB_Album', 'shortcode_callback') );

//Registers options page
add_action('admin_menu', array('FB_Album', '_setup_opt_menu'));

//Registers actions
register_activation_hook(__FILE__, array('FB_Album', 'init'));

class FB_Album {
	
	protected static $album_url;	//Stores the album URL in the class
	protected static $album_id; 	//Stores the album ID in the class
	protected static $css_loaded = false;	//Attempt to prevent CSS from being enqueued too many times.
	
	//The graph API URl the plugin will access. Script replaces {ALBUM_ID} with self::$album_id
	//protected static $graph_url = 'https://graph.facebook.com/{ALBUM_ID}?fields=id,name,photos.limit().fields(source,link,images,name)';
	protected static $graph_url = 'https://graph.facebook.com/{ALBUM_ID}?fields=id,name,photos.limit({LIMIT}).fields(source,link,images,name)';
	protected static $graph_count_url = 'https://graph.facebook.com/{ALBUM_ID}?fields=count';
	
	//Graph API plugin accesses through widget, limits query to just 6 items.
	protected static $widget_graph_url = 'https://graph.facebook.com/{ALBUM_ID}?fields=id,name,photos.limit({LIMIT}).fields(source,link,images,name)';
	
	//Init, let us save options
	public static function init() {
		add_option('fbalbum_size', '5');
	}
	
	/**
	 * Short code callback for WordPress add_shortcode
	 * @return void
	 */
	public static function shortcode_callback( $atts ) {
		
		//If CSS not loaded yet, enqueue it. Allows people to use multiple shortcodes without multiple enqueues.
		if(!self::$css_loaded)
			wp_enqueue_style('fbalbum-style', plugins_url( '/fbalbum.css', __FILE__ ));
		
		//If no URL entered, defaults to Facebook Timeline album
		extract( shortcode_atts( array(
			'url' => '',
		), $atts ) );
		
		self::_set_album_url( $url );
		//Sets up album HTML. Gathers data from class variables.
		return self::print_album();		
	}
	
	protected static function _get_graph_count() {
			$graph_count_url = str_replace('{ALBUM_ID}', self::_get_album_id(), self::$graph_count_url);
			$graph_count	= @file_get_contents($graph_count_url);
			$graph_count	= json_decode($graph_count, true);
			if($graph_count > 200) {
				$graph_count = 200;
			}
			return $graph_count['count'];
	}

	public static function _get_graph_url($limit) {
			$facebook_graph_url = str_replace('{ALBUM_ID}', self::_get_album_id(), self::$graph_url);
			$facebook_graph_url = str_replace('{LIMIT}', $limit, $facebook_graph_url);
			if(ini_get('allow_url_fopen')) {
				$facebook_graph_results = @file_get_contents($facebook_graph_url);
			} else {
				//Attempt CURL
				if (extension_loaded('curl')){
					//CURL REST API to get users recent photos
					$ch = curl_init();
				
					//set the url, number of POST vars, POST data
					curl_setopt($ch,CURLOPT_URL, $facebook_graph_url);
					curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
					$facebook_graph_results = curl_exec($ch);
					curl_close($ch);
					
				} else {
					die('Sorry, your server does not allow remote fopen or have CURL');
				}
			}
			$facebook_graph_results = json_decode($facebook_graph_results, true);
			return $facebook_graph_results;
	}
	
	public static function print_album() {
			if(!self::_get_album_id() || !($fb = self::_get_graph_url(self::_get_graph_count())) )
				return 'Sorry, there was an error loading the Facebook album, please refresh the page and try again.';
						
			
			
			$html = '<h2><a href="' . self::_clean_url(self::_get_album_url()) . '" target="_blank"">' . $fb['name'] . '</a></h2>';
			
			//Facebook uses multiple image sizes, I chose array slot 5, which is 320x320. Unhide the var_dump below to see other options.
			//var_dump($fb['photos']['data'][1]); 
			
			if(!$fb['photos'])
				return 'Album not found, or is not public.';
				
			self::_enqueue_resources();
			
			$size = get_option('fbalbum_size', '5');
			
			foreach ($fb['photos']['data'] as $img) {
				
				$html .= '<div class="fbalbum fbalbum-wrapper">';
				$html 	.= '<a href="'. self::_clean_url($img['source']) . '" title="'. $img['name'] .'" target="_blank" rel="lightbox">';
				$html 	.= '<div class="image size-'. $size .'" style="background: url(' . self::_clean_url($img['images'][$size]['source']) . ')">&nbsp;</div>';
				$html	.= '</a>';
				$html .= '</div>';
			}
			$html .= '<div style="clear:both">&nbsp;</div>';
			return $html;
	}

	protected static function _enqueue_resources() {
			wp_enqueue_script('jquery');
			wp_enqueue_script('lightbox-js', plugins_url( '/lightbox/lightbox.js', __FILE__ ));	
			wp_enqueue_style('lightbox-style', plugins_url( 'lightbox/lightbox.css', __FILE__ ));
	}

	 public static function _setup_opt_menu() {
		add_options_page('Facebook Album', 'Facebook Album', 'manage_options', 'fbalbum-menu', array('FB_Album', '_build_opt_menu'), '', 25);
	}
	 
	public static function _build_opt_menu() {
		//Setup plugin options.
		$thumb_size = get_option('fbalbum_size');
		
		//If nonce, do update
		if(isset($_POST["fbalbum_nonce"])):
			$thumb_size = $_POST["fbalbum_size"];
			update_option('fbalbum_size', $thumb_size);
			echo '<div class="updated"><p><strong>Facebook Albums has been updated</strong></p></div>';	
		endif; ?>
		
			<div class="wrap">
				<div id="icon-options-general" class="icon32"><br></div>
				<h2>Facebook Album Options</h2>
				<p>Pick your thumbnail size to display on pages. More options and features coming soon. Suggestions? Send an email to nmd.matt@gmail.com on features you would like.</p>
				
				<form method="post" action="" name="fbalbum_options">
					<?php 	wp_nonce_field( basename( __FILE__ ), 'fbalbum_nonce' ); ?>
					<table class="form-table">
						<tbody>
							<tr align="top">
								<th scope="row"><label for="">Thumbnail Size</label></th>
								<td><select name="fbalbum_size" id="fbalbum_size" value="" class="regular-text">
										<option value="8" <?php if($thumb_size == 8) echo 'selected="selected"'; ?>>Smaller</option>
							  			<option value="6" <?php if($thumb_size == 6) echo 'selected="selected"'; ?>>Small</option>
							  			<option value="5" <?php if($thumb_size == 5) echo 'selected="selected"'; ?>>Medium</option>
							  			<option value="4" <?php if($thumb_size == 4) echo 'selected="selected"'; ?>>Big</option>
							  			<option value="3" <?php if($thumb_size == 3) echo 'selected="selected"'; ?>>Bigger</option>
							  			<option value="2" <?php if($thumb_size == 2) echo 'selected="selected"'; ?>>Large</option>
									<p class="description">Pick a thumbnail size to be displayed on shortcode pages.</p></td>
							</tr>
						</tbody>
					</table>
					  <p class="submit">
					  <input type="submit" name="Submit" class="button-primary" value="<?php esc_attr_e('Save Changes') ?>" />
					  </p>
				</form>
					
				</div>
			</div>

		<?php
		
	}
	
	/**
	 * Finds and saves album ID by breaking apart the Facebook URL
	 * @return void
	 */
	protected static function _find_album_id() {
		if(!self::_get_album_url())
				return;
		
		//Explodes URL based on slashes, we need the end of the URL
		$facebook_album_id = explode('/', self::_get_album_url());
		$facebook_album_id = $facebook_album_id['5'];
		//Explodes section by periods, Album ID is first of the 3 sets of numbers
		$facebook_album_id = explode('.', $facebook_album_id);
		$facebook_album_id = $facebook_album_id['1'];
		
		self::_set_album_id( $facebook_album_id );
	}
	
	/**
	 * Sets $album_url within class.
	 * @param url
	 * @return void
	 */
	public static function _set_album_url( $url ) {
		 self::$album_url = $url; 
		 self::_find_album_id();
	}
	/**
	 * Gets $album_url from within class
	 * @return url
	 */
	 public static function _get_album_url() { return self::$album_url; }
	 /**
	  * Sets $album_id within class
	  * @param id
	  * @return void
	  */
	  public static function _set_album_id( $id ) { self::$album_id = $id; }
	  /**
	   * Gets $album_url from within class
	   */
	   public static function _get_album_id() { return self::$album_id; }
	  /**
	   * Allows for access to the widget graph API URL, only used in widget.
	   * @return string
	   */
	   public static function _get_widget_graph_url() { return self::$widget_graph_url; }
	   /**
	    * Checks if CSS is loaded or not, versus calling variable directly within class
	    * @return boolean
	    */
	   public static function _check_css() { return self::$css_loaded; }
		
		/**
		 * Makes URLs validator friendly by replacing & to &amp;
		 * @param url
		 * @return string
		 */
		public static function _clean_url( $url ) {
			return str_replace('&', '&amp;', $url);
		}
		/**
		 * Switch on sizes for CSS.
		 */
}


/**
 * The Facebook Album Widget
 * Displays 6 
 */
class FB_Album_Widget extends WP_Widget {

  public function __construct() {
    parent::__construct( 'facebook-album', 'Facebook Album', array('description' => 'Paste a Facebook Page album URL and display the most recent photos as a widget.') );
		//Widget has been set up
	}

 	public function form( $instance ) {
 		
		//The form you see on the admin side
    	$instance = wp_parse_args( (array) $instance, array( 'url' => '', 'thumb_size' => '', 'limit' => '6' ) );
    	$url = $instance['url'];
		$thumb_size = $instance['thumb_size'];
		$limit = $instance['limit'];
		?>
	  	<p><label for="<?php echo $this->get_field_id('url'); ?>">Facebook Album URL: <input class="widefat" id="<?php echo $this->get_field_id('url'); ?>" name="<?php echo $this->get_field_name('url'); ?>" type="text" value="<?php echo attribute_escape($url); ?>" /></label></p>
	  	<p><label for="<?php echo $this->get_field_id('thumb_size'); ?>">Thumbnail Size:</label><select id="<?php echo $this->get_field_id('thumb_size'); ?>" name="<?php echo $this->get_field_name('thumb_size'); ?>">
	  			<option value="8" <?php if($thumb_size == 8) echo 'selected="selected"'; ?>>Smaller</option>
	  			<option value="6" <?php if($thumb_size == 6) echo 'selected="selected"'; ?>>Small</option>
	  			<option value="5" <?php if($thumb_size == 5) echo 'selected="selected"'; ?>>Medium</option>
	  			<option value="4" <?php if($thumb_size == 4) echo 'selected="selected"'; ?>>Big</option>
	  			<!--<option value="3" <?php if($thumb_size == 3) echo 'selected="selected"'; ?>>Bigger</option>-->
	  		</select>
	  	<p><label for="<?php echo $this->get_field_id('limit'); ?>">Number of pictures: <input id="<?php echo $this->get_field_id('limit'); ?>" name="<?php echo $this->get_field_name('limit'); ?>" type="text" value="<?php echo attribute_escape($limit); ?>" size="3"/></label></p>
	  	</p>
		<?php		
	}

	public function update( $new_instance, $old_instance ) {
		//Saves the options
		
		    $instance = $old_instance;
		    $instance['url'] = $new_instance['url'];
			$instance['thumb_size'] = $new_instance['thumb_size'];
			$instance['limit'] = $new_instance['limit'];
		    return $instance;
	}

	public function widget( $args, $instance ) {
		extract($args);
		
		echo $before_widget;
		
		if($instance['url'] != '') {
			FB_Album::_set_album_url( $instance['url']);
		} else { 
			echo 'No Facebook Album specified.';
			return;
		}
	
		if(!FB_Album::_get_album_id()) {
			echo 'Sorry, there was an error loading the Facebook album, please refresh the page and try again.';
			return;
		} else {

			//If CSS not loaded yet, enqueue it. Allows people to use multiple shortcodes without multiple enqueues.
			if(!FB_Album::_check_css())
				wp_enqueue_style('fbalbum-style', plugins_url( '/fbalbum.css', __FILE__ ));
			
			/*$facebook_graph_url = str_replace('{ALBUM_ID}', FB_Album::_get_album_id(), FB_Album::_get_widget_graph_url());
			$facebook_graph_url = str_replace('{LIMIT}', $instance['limit'], $facebook_graph_url);
			//$facebook_graph_results = @file_get_contents($facebook_graph_url);
			$facebook_graph_results = json_decode($facebook_graph_results, true);
			$fb = $facebook_graph_results;*/
			
			if(!FB_Album::_get_album_id() || !($fb = FB_Album::_get_graph_url($instance['limit'])) )
				return 'Sorry, there was an error loading the Facebook album, please refresh the page and try again.';
			
			
			
			if(!$fb['photos']) {
				echo 'Album not found, or is not public.';
			} else {   ?> 
				<?php if( $title ): 
					echo '<h2><a href="' . FB_Album::_clean_url(FB_Album::_get_album_url()) . '" target="_blank"">' . $fb['name'] . '</a></h2>';
				endif; ?>
				<aside class="fbalbum fbalbum-widget"> <?php
				
				foreach ($fb['photos']['data'] as $img) { 
					$img_info = $img['images'][$instance['thumb_size']];
					
					?>
					<div class="item">
						<a href="<?php echo FB_Album::_clean_url($img['link']) ?>" target="_blank" rel="lightbox">
							<div class="image size-<?php echo $instance['thumb_size']; ?>" style="background: url('<?php echo FB_Album::_clean_url($img_info['source']) ?>');">&nbsp;</div>
						</a>
					</div>
				
					<?php
				}
				echo '</aside>';
			}
		?>			 <?php 	
		}
		echo $after_widget; 
	}

}
add_action( 'widgets_init', create_function( '', 'return register_widget( "FB_Album_Widget" );' ) );
?>